@@ -80,7 +80,8 @@ sub create {
         my $attr = $attribs{$key};
         my ( $val, $msg ) = $self->run_validation_for_column(
             column => $key,
-            value  => $attribs{$key}
+            value  => $attribs{$key},
+            extra  => [\%attribs],
         );
         if ( not $val ) {
             $self->log->error("There was a validation error for $key");
@@ -249,7 +250,7 @@ sub _guess_table_name {
 
     # Add plugin table prefix if a plugin model
     my $class = ref($self) ? ref($self) : $self;
-    my $app_plugin_root = Jifty->app_class('Plugin');
+    my $app_plugin_root = Jifty->app_class({require => 0}, 'Plugin');
     if ( $class =~ /^(?:Jifty::Plugin::|$app_plugin_root)/ ) {
 
         # Guess the plugin class name
@@ -435,11 +436,13 @@ sub current_user_can {
         # Otherwise, no instruction from the handlers, move along...
     }
 
-    # Abort! Return false for safety
+    # Abort! Return false for safety if the hook exploded
     else {
         return 0;
     }
 
+
+    Carp::confess unless ( $self->current_user );
     if (   $self->current_user->is_bootstrap_user
         or $self->current_user->is_superuser )
     {
@@ -560,7 +563,7 @@ Overrides L<Jifty::DBI::Record> to check the delete ACL.
 sub delete {
     my $self = shift;
     unless ( $self->check_delete_rights(@_) ) {
-        Jifty->log->logcluck("Permission denied");
+        $self->log->logcluck("Permission denied");
         return ( 0, _('Permission denied') );
     }
     $self->SUPER::delete(@_);
@@ -662,6 +665,24 @@ sub printable_table_schema {
     return $schema_gen->create_table_sql_text;
 }
 
+
+=head2 table_schema_statements
+
+When called, this method will generate the SQL schema statements
+for the current version of this class and return it as array.
+
+=cut
+
+sub table_schema_statements {
+    my $class = shift;
+
+    my $schema_gen = $class->_make_schema();
+    return $schema_gen->create_table_sql_statements;
+}
+
+
+
+
 =head2 create_table_in_db
 
 When called, this method will generate the SQL schema for the current
@@ -673,10 +694,8 @@ open database.
 sub create_table_in_db {
     my $class = shift;
 
-    my $schema_gen = $class->_make_schema();
-
     # Run all CREATE commands
-    for my $statement ( $schema_gen->create_table_sql_statements ) {
+    for my $statement ( $class->table_schema_statements ) {
         my $ret = Jifty->handle->simple_query($statement);
         $ret or die "error creating table $class: " . $ret->error_message;
     }
@@ -827,4 +846,81 @@ sub schema_version {
     }
 }
 
+=head2 jifty_serialize_format
+
+This is used to create a hash reference of the object's values. Unlike
+Jifty::DBI::Record->as_hash, this won't transform refers_to columns into JDBI
+objects. Override this if you want to include calculated values (for use in,
+say, your REST interface)
+
+=cut
+
+sub jifty_serialize_format {
+    my $record = shift;
+    my %data;
+
+    # XXX: maybe just test ->virtual?
+    for ($record->readable_attributes) {
+        next if UNIVERSAL::isa($record->column($_)->refers_to,
+                               'Jifty::DBI::Collection');
+        next if $record->column($_)->container;
+
+        $data{$_} = Jifty::Util->stringify($record->_value($_));
+    }
+
+    return \%data;
+}
+
+=head2 autogenerate_action
+
+Controls which of the L<Jifty::Action::Record> subclasses are
+automatically set up for this model; this subroutine is passed one of
+the strings C<Create>, C<Update>, C<Delete>, C<Search> or C<Execute>, and should
+return a true value if that action should be autogenerated.
+
+The default method returns 0 for all action classes if the model is
+marked as L</is_private>.  It returns 0 for all actions that are not
+C<Search> if the model is marked as L</is_protected>; otherwise, it
+returns true.
+
+=cut
+
+sub autogenerate_action {
+    my $class = shift;
+    my($action) = @_;
+
+    return 0 if $class->is_private;
+    return 0 if $class->is_protected and $action ne "Search";
+
+    return 1;
+}
+
+=head2 is_private
+
+Override this method to return true to not generate any actions for
+this model, and to hide it from REST introspection.
+
+=cut
+
+sub is_private { 0 }
+
+=head2 is_protected
+
+Override this method to return true to only generate Search actions
+for this model.
+
+=cut
+
+sub is_protected { return shift->is_private }
+
+=head2 enumerable
+
+Controls whether autogenerated actions with columns that refer to this
+class should attempt to provide a drop-down of possible values or not.
+This method will be called as a class method, and defaults to true.
+
+=cut
+
+sub enumerable { 1 }
+
 1;
