@@ -4,8 +4,9 @@ use warnings;
 package Jifty::Test::WWW::Mechanize;
 use base qw/Test::WWW::Mechanize/;
 
-$ENV{'http_proxy'} = ''; # Otherwise Test::WWW::Mechanize tries to go through your HTTP proxy
+delete $ENV{'http_proxy'}; # Otherwise Test::WWW::Mechanize tries to go through your HTTP proxy
 
+use Test::More;
 use Jifty::YAML;
 use HTML::Lint;
 use Test::HTML::Lint qw();
@@ -49,7 +50,7 @@ sub new {
 Finds the moniker of the first action of type I<ACTION> whose
 "constructor" field I<FIELD1> is I<VALUE1>, and so on.
 
-   my $mon =$mech->moniker_for('BTDT::Action::UpdateTask');
+   my $mon =$mech->moniker_for('MyApp::Action::UpdateInfok');
 
 If there is only one action of type ACTION, be sure not to pass
 any more arguments to this method, or the method will return undef.
@@ -81,7 +82,8 @@ sub moniker_for {
         my $moniker = $1;
 
         for my $id (keys %args) {
-          my $idfield = $f->find_input("J:A:F:F-$id-$moniker");
+          my $idfield = $f->find_input("J:A:F:F-$id-$moniker")
+                     || $f->find_input("J:A:F-$id-$moniker");
           next INPUT unless $idfield and $idfield->value eq $args{$id};
         }
 
@@ -431,8 +433,56 @@ sub follow_link_ok {
         $Test::Builder::Level++;
         Test::HTML::Lint::html_ok( $lint, $self->content );
     }
-} 
+}
 
+=head2 warnings_like WARNING, [REASON]
+
+Tests that the warnings generated by the server (since the last such
+check) match the given C<WARNING>, which should be a regular
+expression.  If an array reference of regular expressions is passed as
+C<WARNING>, checks that one warning per element was received.
+
+=cut
+
+sub warnings_like {
+    my $self = shift;
+    my @args = shift;
+    @args = @{$args[0]} if ref $args[0] eq "ARRAY";
+    my $reason = pop || "Server warnings matched";
+
+    local $Test::Builder::Level = $Test::Builder::Level;
+    $Test::Builder::Level++;
+
+    my $plugin = Jifty->find_plugin("Jifty::Plugin::TestServerWarnings");
+    my @warnings = $plugin->decoded_warnings($self->uri);
+    my $max = @warnings > @args ? $#warnings : $#args;
+    for (0 .. $max) {
+        like($warnings[$_], $_ <= $#args ? qr/$args[$_]/ : qr/(?!unexpected)unexpected warning/, $reason);
+    }
+}
+
+=head2 no_warnings_ok [REASON]
+
+Checks that no warnings were generated by the server (since the last
+such check).
+
+=cut
+
+sub no_warnings_ok {
+    my $self = shift;
+    my $reason = shift || "no warnings emitted";
+
+    local $Test::Builder::Level = $Test::Builder::Level;
+    $Test::Builder::Level++;
+
+    my $plugin   = Jifty->find_plugin("Jifty::Plugin::TestServerWarnings");
+    my @warnings = $plugin->decoded_warnings( $self->uri );
+
+    is( @warnings, 0, $reason );
+    for (@warnings) {
+        diag("got warning: $_");
+    }
+}
 
 =head2 session
 
