@@ -21,6 +21,7 @@ use Params::Validate qw(:all);
 use HTML::Mason::Exceptions;
 use HTML::Mason::FakeApache;
 use Encode qw();
+use Jifty::View::Mason::Request;
 
 use Class::Container;
 use base qw(Jifty::View Class::Container);
@@ -44,19 +45,21 @@ __PACKAGE__->contained_objects
 =head2 new PARAMHASH
 
 Takes a number of key-value parameters; see L<HTML::Mason::Params>.
-Defaults the C<out_method> to L</out_method>, and the C<request_class>
-to L<HTML::Mason::Request::Jifty> (below).  Finally, adds C<h> and
-C<u> escapes, which map to L</escape_uri> and L<escape_utf8>
-respectively.
+Defaults the C<out_method> to appending to L<Jifty::Handler/buffer>
+and the C<request_class> to L<Jifty::View::Mason::Request> (below).
+Finally, adds C<h> and C<u> escapes, which map to L</escape_uri> and
+L<escape_utf8> respectively.
 
 =cut
 
 sub new {
     my $package = shift;
 
+    $package->create_cache_directories;
+
     my %p = @_ || $package->config;
-    my $self = $package->SUPER::new( request_class => 'HTML::Mason::Request::Jifty',
-                                     out_method => $package->can('out_method'),
+    my $self = $package->SUPER::new( request_class => 'Jifty::View::Mason::Request',
+                                     out_method => sub {Carp::cluck("Mason output skipped Jifty's output stack!") if grep {defined and length} @_},
                                      %p );
     $self->interp->compiler->add_allowed_globals('$r');
     $self->interp->set_escape( h => \&escape_utf8 );
@@ -78,6 +81,8 @@ mode.
 =cut
 
 sub config {
+    my $self = shift;
+
     my %config = (
         static_source => 1,
         use_object_files => 1,
@@ -99,15 +104,14 @@ sub config {
     );
 
     my $root_serial = 0;
+    my %seen; $seen{$_} = 1 for map Jifty->config->framework('Web')->{$_}, qw/TemplateRoot DefaultTemplateRoot/;
     for my $plugin (Jifty->plugins) {
         my $comp_root = $plugin->template_root;
-        unless  ( $comp_root and -d $comp_root) {
-            next;
-        }
-        Jifty->log->debug( "Plugin @{[ref($plugin)]} mason component root added: (@{[$comp_root ||'']})");
+        next unless ( defined $comp_root and -d $comp_root and not $seen{$comp_root}++);
+        $plugin->log->debug( "Plugin @{[ref($plugin)]} mason component root added: (@{[$comp_root ||'']})");
         push @{ $config{comp_root} }, [ ref($plugin)."-". $root_serial++ => $comp_root ];
     }
-    push @{$config{comp_root}}, [jifty => Jifty->config->framework('Web')->{'DefaultTemplateRoot'}];
+    push @{$config{comp_root}}, [jifty => Jifty::Util->absolute_path( Jifty->config->framework('Web')->{'DefaultTemplateRoot'})];
 
     # In developer mode, we want halos, refreshing and all that other good stuff. 
     if (Jifty->config->framework('DevelMode') ) {
@@ -115,6 +119,10 @@ sub config {
         $config{static_source}    = 0;
         $config{use_object_files} = 0;
     }
+
+    # We require autoflush now.
+    $config{autoflush} = 1;
+
     return %config;
 }
 
@@ -156,14 +164,30 @@ sub escape_uri {
 
 =head2 template_exists COMPONENT
 
-A convenience method for $self->interp->comp_exists().  
-(Jifty uses this method as part of its standard Templating system API).
+Checks if the C<COMPONENT> exists, or if C<COMPONENT/index.html>
+exists, and returns which one did.  If neither did, it seaches for
+C<dhandler> components which could match, returning C<COMPONENT> if it
+finds one.  Finally, if it finds no possible component matches,
+returns undef.
+
+Note that this algorithm does not actually decisively return if Mason
+I<will> handle a given component; the I<dhandler>s could defer
+handling, for instance.
 
 =cut
 
 sub template_exists {
-	my $self = shift;
-	return $self->interp->comp_exists(@_);
+    my $self = shift;
+    my ($component) = @_;
+    $component =~ s{^/*}{/};
+    return $component if $self->interp->comp_exists($component);
+    return "$component/index.html" if $self->interp->comp_exists("$component/index.html");
+
+    my $dhandler = $self->interp->dhandler_name;
+    $dhandler = "dhandler" unless defined $dhandler;
+    return if defined $dhandler and not length $dhandler;
+    return $component if $self->interp->find_comp_upwards($component, $dhandler);
+    return undef;
 }
 
 
@@ -197,30 +221,7 @@ sub handle_comp {
     $self->interp->set_global('$jifty_internal_request', 1) if defined $args;
 
     my %args = $args ? %$args : $self->request_args($r);
-
-    my @result;
-    if (wantarray) {
-        @result = eval { $self->interp->exec($comp, %args) };
-    } elsif ( defined wantarray ) {
-        $result[0] = eval { $self->interp->exec($comp, %args) };
-    } else {
-        eval { $self->interp->exec($comp, %args) };
-    }
-
-    if (my $err = $@) {
-        my $retval = isa_mason_exception($err, 'Abort')   ? $err->aborted_value  :
-                     isa_mason_exception($err, 'Decline') ? $err->declined_value :
-                     rethrow_exception $err;
-
-        # Unlike under mod_perl, we cannot simply return a 301 or 302
-        # status and let Apache send headers, we need to explicitly
-        # send this header ourself.
-        $r->send_http_header if $retval && grep { $retval eq $_ } ( 200, 301, 302 );
-
-        return $retval;
-    }
-
-    return wantarray ? @result : defined wantarray ? $result[0] : undef;
+    $self->interp->exec($comp, %args);
 }
 
 =head2 request_args
@@ -234,74 +235,18 @@ sub request_args {
     return %{Jifty->web->request->arguments}, %{Jifty->web->request->template_arguments || {}};
 }
 
-###########################################################
-package HTML::Mason::Request::Jifty;
-# Subclass for HTML::Mason::Request object $m
 
-=head1 HTML::Mason::Request::Jifty
+=head2 create_cache_directories
 
-Subclass of L<HTML::Mason::Request> which is customised for Jifty's use.
+Attempts to create our app's mason cache directory.
 
 =cut
 
-use HTML::Mason::Exceptions;
-use HTML::Mason::Request;
-use base qw(HTML::Mason::Request);
-
-=head2 auto_send_headers
-
-Doesn't send headers if this is a subrequest (according to the current
-L<Jifty::Request>).
-
-=cut
-
-sub auto_send_headers {
-    Jifty::View->auto_send_headers;
-}
-
-=head2 exec
-
-Actually runs the component; in case no headers have been sent after
-running the component, and we're supposed to send headers, sends them.
-
-=cut
-
-sub exec
-{
-    my $self = shift;
-    my $r = Jifty->handler->apache;
-    my $retval;
-
-    eval { $retval = $self->SUPER::exec(@_) };
-
-    if (my $err = $@)
-    {
-	$retval = isa_mason_exception($err, 'Abort')   ? $err->aborted_value  :
-                  isa_mason_exception($err, 'Decline') ? $err->declined_value :
-                  rethrow_exception $err;
+sub create_cache_directories {
+    for ( Jifty->config->framework('Web')->{'DataDir'} ) {
+        Jifty::Util->make_path( Jifty::Util->absolute_path($_) );
     }
-
-    # On a success code, send headers if they have not been sent and
-    # if we are the top-level request. Since the out_method sends
-    # headers, this will typically only apply after $m->abort.
-    if ($self->auto_send_headers
-        and not $r->http_header_sent
-        and (!$retval or $retval==200)) {
-        $r->send_http_header();
-    }
-}
-
-=head2 redirect
-
-Calls L<Jifty::Web/redirect>.
-
-=cut
-
-sub redirect {
-    my $self = shift;
-    my $url = shift;
-
-    Jifty->web->redirect($url);
 }
 
 1;
+
