@@ -183,6 +183,7 @@ Renders the opening form tag.
 
 sub start {
     my $self = shift;
+    local $Log::Log4perl::caller_depth = $Log::Log4perl::caller_depth + 1;
 
     my %args = (@_);
 
@@ -199,7 +200,9 @@ sub start {
         }
     }
 
-    my $form_start = qq!<form method="post" action="!  . Jifty->web->escape( $self->submit_to || $ENV{PATH_INFO}) . qq!"!;
+    my $root = $self->submit_to;
+    ($root) = $ENV{'REQUEST_URI'} =~ /([^\?]*)/ unless defined $root;
+    my $form_start = qq!<form method="post" action="!  . Jifty->web->escape( $root ) . qq!"!;
     $form_start .= qq! name="@{[ $self->name ]}"! if defined $self->name;
     $form_start .= qq! target="@{[ $self->target ]}"! if defined $self->target;
     $form_start .= qq! autocomplete="off"!  if defined $self->disable_autocomplete;
