@@ -110,14 +110,23 @@ region that the region was given when it was last rendered.
 
 =item effect => STRING
 
-The Scriptaculous visual effect to use when updating or creating the
-fragment.
+The Scriptaculous or jQuery visual effect to use when updating or
+creating the fragment.
 
 =item effect_args => HASHREF
 
 A hashref of arguments to pass to the effect when it is created.  These
 can be used to change the duration of the effect, for instance.
 
+=item remove_effect => STRING
+
+As C<effect>, but for when the previous version of the region is
+removed.
+
+=item remove_effect_args => HASHREF
+
+As C<effect_args>, but for C<remove_effect>.
+
 =item beforeclick => STRING
 
 String contains some Javascript code to be used before a click.
@@ -153,10 +162,13 @@ error from your browser.
 
 =cut
 
-sub handlers { qw(onclick onchange ondblclick onmousedown onmouseup onmouseover 
-                  onmousemove onmouseout onfocus onblur onkeypress onkeydown 
-                  onkeyup onselect) }
+use constant handlers => qw(
+    onclick onchange ondblclick onmousedown onmouseup onmouseover
+    onmousemove onmouseout onfocus onblur onkeypress onkeydown
+    onkeyup onselect
+);
 
+use constant possible_handlers => { map {($_ => 1)} handlers};
 
 
 =head2 accessors
@@ -167,11 +179,10 @@ C<new> parameter hash.
 
 =cut
 
-sub accessors { shift->handlers, qw(class title key_binding key_binding_label id label tooltip continuation rel) }
-__PACKAGE__->mk_accessors(qw(_onclick _onchange _ondblclick _onmousedown _onmouseup _onmouseover 
-                             _onmousemove _onmouseout _onfocus _onblur _onkeypress _onkeydown 
-                             _onkeyup _onselect
-                             class title key_binding key_binding_label id label tooltip continuation rel));
+sub accessors { handlers, qw(class title key_binding key_binding_label id label tooltip continuation rel) }
+
+__PACKAGE__->mk_accessors(map "_$_", handlers);
+__PACKAGE__->mk_accessors(qw(class title key_binding key_binding_label id label tooltip continuation rel) );
 
 =head2 new PARAMHASH OVERRIDE
 
@@ -181,204 +192,111 @@ PARAMHASH, and set with accessors for the hash values in OVERRIDE.
 =cut
 
 sub new {
-    my ($class, $args, $override) = @_;
+    my ($class, $args, $other) = @_;
+    $args = {%{$args}, %{$other || {}}};
     # force using accessor for onclick init
-    for my $trigger ( __PACKAGE__->handlers() ) {
-      if (my $triggerArgs = delete $args->{$trigger}) {
-        $override->{$trigger} = $triggerArgs;
-      }
-    }
+    my $override = {};
+    $override->{$_} = delete $args->{$_}
+        for grep {possible_handlers->{$_} and defined $args->{$_}} keys %{$args};
 
     my $self = $class->SUPER::new($args);
 
-    if ($override) {
-        for my $field ( $self->accessors() ) {
-            # XXX: warn about unexpected ones
-            $self->$field( $override->{$field} ) if exists $override->{$field};
-        }
-    }
+    $self->{handlers_used} = {};
+    $self->$_( $override->{$_} ) for keys %{$override};
 
     return $self;
 }
 
+__PACKAGE__->_mk_normalising_accessor($_) for __PACKAGE__->handlers;
+
+sub _mk_normalising_accessor {
+    my ($class, $accessor) = @_;
+    my $internal_method = "_$accessor";
+    no strict 'refs';
+    *{$accessor} = sub {
+        my $self = shift;
+        return $self->{$internal_method} unless @_;
+
+        $self->$internal_method($self->_handler_setup($internal_method, @_));
+    };
+}
 
- 
 =head2 onclick
 
 The onclick event occurs when the pointing device button is clicked 
 over an element. This attribute may be used with most elements.
 
-=cut
-
-sub onclick {
-    my $self = shift;
-    $self->_onclick($self->_handler_setup('_onclick', @_));
-}
-
 =head2 onchange
 
 The onchange event occurs when a control loses the input focus 
 and its value has been modified since gaining focus. This handler 
 can be used with all form elements.
 
-=cut
-
-sub onchange {
-    my $self = shift;
-    $self->_onchange($self->_handler_setup('_onchange', @_));
-}
-
-
-
 =head2 ondblclick
 
 The ondblclick event occurs when the pointing device button is 
 double clicked over an element.  This handler 
 can be used with all form elements.
 
-=cut
-
-sub ondblclick {
-    my $self = shift;
-    $self->_ondblclick($self->_handler_setup('_ondblclick', @_));
-}
-
 =head2 onmousedown
 
 The onmousedown event occurs when the pointing device button is 
 pressed over an element.  This handler 
 can be used with all form elements.
 
-=cut
-
-sub onmousedown {
-    my $self = shift;
-    $self->_onmousedown($self->_handler_setup('_onmousedown', @_));
-}
-
 =head2 onmouseup
 
 The onmouseup event occurs when the pointing device button is released 
 over an element.  This handler can be used with all form elements.
 
-=cut
-
-sub onmouseup {
-    my $self = shift;
-    $self->_onmouseup($self->_handler_setup('_onmouseup', @_));
-}
-
 =head2 onmouseover
 
 The onmouseover event occurs when the pointing device is moved onto an 
 element.  This handler can be used with all form elements.
 
-=cut
-
-sub onmouseover {
-    my $self = shift;
-    $self->_onmouseover($self->_handler_setup('_onmouseover', @_));
-}
-
 =head2 onmousemove
 
 The onmousemove event occurs when the pointing device is moved while it 
 is over an element.  This handler can be used with all form elements.
 
-=cut
-
-sub onmousemove {
-    my $self = shift;
-    $self->_onmousemove($self->_handler_setup('_onmousemove', @_));
-}
-
 =head2 onmouseout
 
 The onmouseout event occurs when the pointing device is moved away from 
 an element.  This handler can be used with all form elements.
 
-=cut
-
-sub onmouseout {
-    my $self = shift;
-    $self->_onmouseout($self->_handler_setup('_onmouseout', @_));
-}
-
 =head2 onfocus
 
 The onfocus event occurs when an element receives focus either by the 
 pointing device or by tabbing navigation.  This handler 
 can be used with all form elements.
 
-=cut
-
-sub onfocus {
-    my $self = shift;
-    $self->_onfocus($self->_handler_setup('_onfocus', @_));
-}
-
 =head2 onblur
 
 The onblur event occurs when an element loses focus either by the pointing 
 device or by tabbing navigation.  This handler can be used with all 
 form elements.
 
-=cut
-
-sub onblur {
-    my $self = shift;
-    $self->_onblur($self->_handler_setup('_onblur', @_));
-}
-
 =head2 onkeypress
 
 The onkeypress event occurs when a key is pressed and released over an 
 element.  This handler can be used with all form elements.
 
-=cut
-
-sub onkeypress {
-    my $self = shift;
-    $self->_onkeypress($self->_handler_setup('_onkeypress', @_));
-}
-
 =head2 onkeydown
 
 The onkeydown event occurs when a key is pressed down over an element. 
 This handler can be used with all form elements.
 
-=cut
-
-sub onkeydown {
-    my $self = shift;
-    $self->_onkeydown($self->_handler_setup('_onkeydown', @_));
-}
-
 =head2 onkeyup
 
 The onkeyup event occurs when a key is released over an element. 
 This handler can be used with all form elements.
 =cut
 
-sub onkeyup {
-    my $self = shift;
-    $self->_onkeyup($self->_handler_setup('_onkeyup', @_));
-}
-
 =head2 onselect
 
 The onselect event occurs when a user selects some text in a text field. 
 This attribute may be used with the text and textarea fields.
 
-=cut
-
-sub onselect {
-    my $self = shift;
-    $self->_onselect($self->_handler_setup('_onselect', @_));
-}
-
-
-
 =head2 _handler_setup
 
 This method is used by all handlers to normalize all arguments.
@@ -390,6 +308,8 @@ sub _handler_setup {
     my $trigger = shift;
 
     return $self->$trigger unless @_;
+    $trigger =~ s/^_//;
+    $self->{handlers_used}{$trigger} = 1;
 
     my ($arg) = @_;
 
@@ -407,15 +327,13 @@ sub _handler_setup {
 
             my @submit_tmp;
             foreach my $submit ( @{$hook->{submit}}) {
-                if (!ref($submit)){ 
-                        push @submit_tmp, $submit;
-                    } 
-                elsif(blessed($submit)) {
-                        push @submit_tmp, $submit->moniker;
-
+                if (!ref($submit)) {
+                    push @submit_tmp, $submit;
+                } elsif(blessed($submit)) {
+                    push @submit_tmp, $submit->moniker;
                 } else { # it's a hashref
-                        push @submit_tmp, $submit->{'action'}->moniker;
-                        $hook->{'action_arguments'}->{ $submit->{'action'}->moniker } = $submit->{'arguments'};
+                    push @submit_tmp, $submit->{'action'}->moniker;
+                    $hook->{'action_arguments'}->{ $submit->{'action'}->moniker } = $submit->{'arguments'};
                 }
 
             }
@@ -443,21 +361,57 @@ sub _handler_setup {
 
 }
 
-=head2 javascript
+=head2 handlers_used
 
-Returns the javascript necessary to make the events happen.
+Returns the names of javascript handlers which exist for this element.
 
 =cut
+
+sub handlers_used {
+    my $self = shift;
+    return keys %{$self->{handlers_used}};
+}
+
+=head2 javascript
+
+Returns the javascript necessary to make the events happen, as a
+string of HTML attributes.
+
+=cut
+
 sub javascript {
     my $self = shift;
+    my %response = $self->javascript_attrs;
+    return join "", map {qq| $_="| . Jifty->web->escape($response{$_}).qq|"|} sort keys %response;
+}
 
-    my $response = "";
+=head2 javascript_attrs
+
+Returns the javascript necessary to make the events happen, as a
+hash of attribute-name and value.
+
+=cut
+
+sub javascript_attrs {
+    my $self = shift;
+
+    my %response;
 
   HANDLER:
-    for my $trigger ( $self->handlers ) {
-        my $value = $self->$trigger;
+    for my $trigger ( $self->handlers_used ) {
+        # Walk around the Class::Accessor, for speed
+        my $value = $self->{"_$trigger"};
         next unless $value;
 
+        if ( !( $self->handler_allowed($trigger) ) ) {
+            $self->log->error(
+                      "Handler '$trigger' is not supported for field '"
+                    . $self->label
+                    . "' with class "
+                    . ref $self );
+            next;
+        }
+
         my @fragments;
             # if $actions is undef, that means we're submitting _all_ actions in the clickable
             # if $actions is defined but empty, that means we're submitting no actions
@@ -467,19 +421,9 @@ sub javascript {
         my $beforeclick;
         my $action_arguments = {};
         for my $hook (grep {ref $_ eq "HASH"} (@{$value})) {
-
-            if (!($self->handler_allowed($trigger))) {
-                Jifty->log->error("Handler '$trigger' is not supported for field '" . 
-                                  $self->label . 
-                                  "' with class " . ref $self);
-                next HANDLER;
-            }
-
             my %args;
 
             # Submit action
-          
-            
             if ( exists $hook->{submit} ) {
                 $actions = undef;
                 my $disable_form_on_click = exists $hook->{disable} ? $hook->{disable} : 1;
@@ -551,15 +495,15 @@ sub javascript {
             $args{toggle} = 1 if $hook->{toggle};
 
             # Effects
-            $args{$_} = $hook->{$_} for grep {exists $hook->{$_}} qw/effect effect_args/;
+            $args{$_} = $hook->{$_} for grep {exists $hook->{$_}} qw/effect effect_args remove_effect remove_effect_args/;
 
             push @fragments, \%args;
         }
 
-        my $string = join ";", (grep {not ref $_} (ref $value eq "ARRAY" ? @{$value} : ($value)));
+        my $string = join ";", grep {not ref $_} @{$value};
         if ( @fragments or ( !$actions || %$actions ) ) {
 
-            my $update = Jifty->web->escape(
+            my $update =
                 "Jifty.update( "
                     . Jifty::JSON::objToJson(
                     {   actions      => $actions,
@@ -568,8 +512,7 @@ sub javascript {
                         continuation => $self->continuation
                     },
                     { singlequote => 1 }
-                    ) . ", this );"
-            );
+                    ) . ", this );";
             $string
                 .= 'if(event.ctrlKey||event.metaKey||event.altKey||event.shiftKey) return true; '
                 if ( $trigger eq 'onclick' );
@@ -578,14 +521,15 @@ sub javascript {
                 : "$update; return true;";
         }
         if ($confirm) {
-            $string = Jifty->web->escape("if(!confirm(" . Jifty::JSON::objToJson($confirm, {singlequote => 1}) . ")) { Event.stop(event); return false }") . $string;
+            my $text = Jifty::JSON::objToJson($confirm, {singlequote => 1});
+            $string = "if(!confirm($text)){ Jifty.stopEvent(event); return false; }" . $string;
         }
         if ($beforeclick) {
-           $string = Jifty->web->escape($beforeclick) . $string;
+           $string = $beforeclick . $string;
         }
-        $response .= qq| $trigger="$string"|;
+        $response{$trigger} = $string;
     }
-    return $response;
+    return %response;
 }
 
 =head2 javascript_preempt
@@ -659,11 +603,9 @@ sub render_key_binding {
     my $self = shift;
     return unless $self->key_binding;
     Jifty->web->out(
-        '<script type="text/javascript"><!--' .
-        "\n" .
+        '<script type="text/javascript">' .
         Jifty->web->escape($self->key_binding_javascript).
-        "\n" .
-        "--></script>");
+        "</script>");
     return '';
 }
 
