@@ -5,6 +5,10 @@ package Jifty::Web::Form::Field::Collection;
 
 use base qw/Jifty::Web::Form::Field/;
 
+=head1 NAME
+
+Jifty::Web::Form::Field::Collection - render a whole collection of available values
+
 =head2 render_widget
 
 Renders the whole collection of available values.
@@ -51,6 +55,7 @@ sub render_option {
     $id =~ s/\s+/_/;
     my $field = qq! <input type="text" !;
     $field .= qq! name="@{[ $self->input_name ]}"!;
+    $field .= qq! title="@{[ $self->title ]}"! if ($self->title);
     $field .= qq! id="@{[ $id ]}"!;
     $field .= qq! value="0"!;
     $field .= $self->_widget_class;
